module PublisherController
  extend ActiveSupport::Concern

  included do
    before_action :authenticate, only: [:show_all_drafts, :show_liked_learning_objects, :show_liked_collections]
    before_action :set_publisher, only: [:show_all_drafts, :show_all_learning_objects, :show_all_collections, :show_liked_learning_objects, :show_liked_collections]
    before_action -> { authorize @publisher }, only: [:show_all_drafts, :show_liked_learning_objects, :show_liked_collections]
  end

  def show_all_drafts
    render json: LearningObject.where(publisher: @publisher, state: LearningObject.states[:draft])
  end

  # GET /v1/users/1/learning_objects
  def show_all_learning_objects
    render json: LearningObject.where(publisher: @publisher, state: LearningObject.states[:published])
  end

  def show_all_collections
    render json: ::UserPolicy::Scope.new(current_user, @publisher, Collection).resolve.where(owner: @publisher)
  end

  def show_liked_learning_objects
    includes = [:taggings, :tags, :subject_relations, :subjects, :stage_relations, :educational_stages, :publisher, :language, :license]
    render json: LearningObject.includes(includes).find(
      Like.where(user: @publisher, likeable_type: 'LearningObject').pluck(:likeable_id)
    )
  end

  def show_liked_collections
    render json: Collection.find(
      Like.where(user: @publisher, likeable_type: 'Collection').pluck(:likeable_id)
    )
  end

  protected

  def authenticate
    authenticate_user!
  end

  def set_publisher
    user, id = request.path.split('/')[2, 3]
    return nil unless %w(users institutions).include? user
    publisher_model = user.singularize.classify.constantize
    @publisher = publisher_model.find(id)
  end
end
